package io.useless.auth

import java.util.UUID

/**
 * AccessToken defines how requests are authenticated. Every authenticated
 * request must specify an AccessToken guid, e.g. via the Authorization header.
 *
 * An AccessToken MIGHT have a client - this is the account that is actually
 * making the request. If this is missing, then the account is making a request
 * as itself.
 */
trait AccessToken {

  def guid: UUID

  def resourceOwner: Account

  def client: Option[Account]

}

object AccessToken {

  class BaseAccessToken(
    val guid: UUID,
    val resourceOwner: Account,
    val client: Option[Account]
  ) extends AccessToken

  def apply(guid: UUID, resourceOwner: Account, client: Option[Account]): AccessToken = {
    new BaseAccessToken(guid, resourceOwner, client)
  }

}
