package io.useless.client.auth

import java.util.UUID
import scala.concurrent.Future
import scala.concurrent.ExecutionContext.Implicits.global
import play.api.libs.json.{ Json, JsSuccess, JsError }

import io.useless.client._
import io.useless.auth.AccessToken
import io.useless.play.json.AccessTokenJson._

class StandardAuthClient
  extends AuthClient
  with    BaseClient
{

  def getAccessToken(guid: UUID): Future[Option[AccessToken]] = {
    val path = "/auth/access_tokens/%s".format(guid.toString)

    getResponse(path).map { response =>
      response.status match {
        case 200 => {
          Json.fromJson[AccessToken](response.json) match {
            case success: JsSuccess[AccessToken] => Some(success.get)
            case error: JsError => throw new InvalidJsonResponseException(error)
          }
        }
        case 404 => None
        case status: Int => throw new UnexpectedStatusException(status)
      }
    }
  }

  protected def getResponse(path: String) = baseClient.request(path).get

}
