package io.useless.client.login

import java.util.UUID
import scala.concurrent.Future

import io.useless.login.Login
import io.useless.auth.User

class MockLoginClient extends LoginClient {

  def getLogin(email: String, password: String) = {
    val record = records.find { record =>
      record.email == email && record.password == password
    }

    val optLogin = record.map { buildLogin(_) }
    Future.successful(optLogin)
  }

  def createLogin(
    email: String,
    password: String,
    handle: Option[String],
    name: Option[String]
  ) = {
    val result = records.find(_.email == email) match {
      case Some(record: Record) =>
        Left("login for email already exists")
      case None => {
        val record = Record(email, password, handle, name)
        records = records :+ record 
        Right(buildLogin(record))
      }
    }

    Future.successful(result)
  }

  private var records = Seq[Record]()

  private case class Record(
    email: String,
    password: String,
    handle: Option[String],
    name: Option[String]
  )

  private def buildLogin(record: Record) = {
    Login(
      user = new User(UUID.randomUUID, Seq(), record.email, None, None),
      handle = record.handle,
      name = record.name
    )
  }

}
