package io.useless.client.login

import scala.concurrent.Future
import scala.concurrent.ExecutionContext.Implicits.global
import play.api.libs.json.{ Json, JsSuccess, JsError }

import io.useless.client._
import io.useless.login.Login
import io.useless.play.json.LoginJson._

class StandardLoginClient
  extends LoginClient
  with    BaseClient
{

  val path = "/login"

  def getLogin(email: String, password: String): Future[Option[Login]] = {
    getResponse(email, password).map { response =>
      response.status match {
        case 200 => {
          Json.fromJson[Login](response.json) match {
            case success: JsSuccess[Login] => Some(success.get)
            case error: JsError => throw new InvalidJsonResponseException(error)
          }
        }
        case 404 => None
        case status: Int => throw new UnexpectedStatusException(status)
      }
    }
  }

  protected def getResponse(email: String, password: String) = {
    baseClient.request(path).withQueryString("email" -> email, "password" -> password).get
  }

  def createLogin(
    email: String,
    password: String,
    handle: Option[String],
    name: Option[String]
  ): Future[Either[String, Login]] = {
    postResponse(email, password, handle, name).map { response =>
      response.status match {
        case 200 => {
          Json.fromJson[Login](response.json) match {
            case success: JsSuccess[Login] => Right(success.get)
            case error: JsError => throw new InvalidJsonResponseException(error)
          }
        }
        case 422 => Left(response.body)
        case status: Int => throw new UnexpectedStatusException(status)
      }
    }
  }

  protected def postResponse(
    email: String,
    password: String,
    handle: Option[String],
    name: Option[String]
  ) = baseClient.request(path).post(Json.obj(
    "email" -> email,
    "password" -> password,
    "handle" -> handle,
    "name" -> name
  ))

}
