package io.useless.play.authentication

import scala.concurrent.Future
import scala.concurrent.ExecutionContext.Implicits.global

import play.api.mvc._
import play.api.mvc.BodyParsers._

import io.useless.auth.Account

trait ActionComponent {

  class AuthenticatedRequest[A](
    val account: Account,
    private val request: Request[A]
  ) extends WrappedRequest(request)

  def Authenticated[A](bodyParser: BodyParser[A])(action: AuthenticatedRequest[A] => Result): Action[A]

  def Authenticated(f: AuthenticatedRequest[AnyContent] => Result): Action[AnyContent]  = {
    Authenticated(parse.anyContent)(f)
  }

}

trait AsyncActionComponent extends ActionComponent {

  self: Controller with
        AuthenticatorComponent with
        AuthorizerComponent =>

  def Authenticated[A](bodyParser: BodyParser[A])(action: AuthenticatedRequest[A] => Result) = {
    Action(bodyParser) { request =>
      Async {
        authenticator.authenticate(request).flatMap { maybeAccount =>
          maybeAccount.map { account =>
            authorizer.authorize(account).flatMap { authorized =>
              if (authorized) {
                val authenticatedRequest = new AuthenticatedRequest(account, request)
                action(authenticatedRequest) match {
                  case asyncResult: AsyncResult => asyncResult.result
                  case result: Result => Future.successful(result)
                }
              } else {
                Future.successful(Unauthorized)
              }
            }
          }.getOrElse { Future.successful(Unauthorized) }
        }
      }
    }
  }

}
