package io.useless.play.authentication

import java.util.UUID
import scala.concurrent.Future

import io.useless.auth.AccessToken
import io.useless.client.auth.AuthClient

trait AuthDaoComponent {

  def authDao: AuthDao

  trait AuthDao {

    def getAccessToken(guid: UUID): Future[Option[AccessToken]]

  }

}

trait MockAuthDaoComponent extends AuthDaoComponent {

  class MockAuthDao(accessTokens: Seq[AccessToken]) extends AuthDao {

    def getAccessToken(guid: UUID) = {
      Future.successful(accessTokens.find { _.guid == guid })
    }

  }

}

trait ClientAuthDaoComponent extends AuthDaoComponent {

  class ClientAuthDao extends AuthDao {

    private lazy val authClient = AuthClient.instance

    def getAccessToken(guid: UUID) = authClient.getAccessToken(guid)

  }

}
