package io.useless.play.authentication

import scala.concurrent.Future

import play.api.mvc.Controller

import io.useless.auth.Account
import io.useless.auth.Role._

trait Authentication
  extends BaseAuthentication
  with ClientAuthDaoComponent
{

  self: Controller =>

  lazy val authDao = new ClientAuthDao

}

trait BaseAuthentication
  extends AsyncActionComponent

  with    CompositeAuthenticatorComponent
  with    HeaderAuthenticatorComponent
  with    CookieAuthenticatorComponent
  with    QueryParameterAuthenticatorComponent

  with    CompositeAuthorizerComponent
  with    DelegateAuthorizerComponent
  with    RoleAuthorizerComponent
{

  self: Controller with
        AuthDaoComponent =>

  val authenticator: Authenticator = new CompositeAuthenticator(Seq(
    new HeaderAuthenticator("Authorization"),
    new CookieAuthenticator("auth"),
    new QueryParameterAuthenticator("auth")
  ))

  def authorize(account: Account) = Future.successful(true)

  val requiredRoles: Seq[Role] = Seq()

  lazy val authorizer: Authorizer = new CompositeAuthorizer(Seq(
    new DelegateAuthorizer(authorize),
    new RoleAuthorizer(requiredRoles)
  ))

}
