package io.useless.play.authentication

import scala.concurrent.Future
import scala.concurrent.ExecutionContext.Implicits.global

import io.useless.auth.Account
import io.useless.auth.Role._

trait AuthorizerComponent {

  val authorizer: Authorizer

  trait Authorizer {

    def authorize(account: Account): Future[Boolean]

  }

}

trait CompositeAuthorizerComponent extends AuthorizerComponent {

  class CompositeAuthorizer(
    authenticators: Seq[Authorizer]
  ) extends Authorizer {

    def authorize(account: Account) = {
      val futures = authenticators.map(_.authorize(account))
      Future.reduce(futures)(_ && _)
    }

  }

}

trait DelegateAuthorizerComponent extends AuthorizerComponent {

  class DelegateAuthorizer(authorizer: (Account) => Future[Boolean]) extends Authorizer {

    def authorize(account: Account) = authorizer(account)

  }

}

trait RoleAuthorizerComponent extends AuthorizerComponent {

  class RoleAuthorizer(requiredRoles: Seq[Role]) extends Authorizer {

    def authorize(account: Account) = {
      Future.successful { 
        requiredRoles.forall { account.roles.contains(_) }
      }
    }

  }

}
