package io.useless.play.json

import java.util.UUID
import play.api.libs.json._
import play.api.libs.functional.syntax._

import io.useless.auth.User
import io.useless.auth.Role._
import io.useless.play.json.UuidJson._
import RoleJson._

object UserJson {

  implicit val userReads: Reads[User] = (
    (__ \ "guid").read[UUID] ~
    (__ \ "roles").read[Seq[Role]] ~
    (__ \ "user" \ "email").read[String] ~
    (__ \ "user" \ "handle").read[Option[String]] ~
    (__ \ "user" \ "name").read[Option[String]]
  ) { (guid: UUID, roles: Seq[Role], email: String, handle: Option[String], name: Option[String]) =>
    new User(guid, roles, email, handle, name)
  }

  implicit val userWrites = new Writes[User] {
    def writes(user: User): JsValue = {
      Json.obj(
        "guid" -> user.guid,
        "roles" -> user.roles,
        "user" -> Json.obj(
          "email" -> user.email,
          "handle" -> user.handle,
          "name" -> user.name
        )
      )
    }
  }

}
